# -*- coding: utf-8 -*-
from demeter.core import *
import modbus_tk.defines as mdef
import time
import random
import struct
import serial

class Handle(object):

	# 设置数据
	def set(self, master, server):
		#master.set_timeout(600)
		server['server_time'] = int(server['server_time'])
		while(True):
			self.run(master, server)
			if server['server_time'] > 0:
					time.sleep(server['server_time'])

	def run(self, master, server):
		gateway = Demeter.service('common').list('device_gateway', search={'server_id':server['id']})
		if gateway:
			for v in gateway:
				if v['product_id']:
					self.runCode(master, v)

	def runCode(self, master, gateway):
		code = Demeter.service('common').list('setting_modbus_code', search={'product_id':gateway['product_id']})
		if code:
			for v in code:
				if v['register_read_address']:
					try:
						self.setValue(v, master, gateway)
					except:
						pass

	# 发送数据
	def send(self, master, server, type_info, info, value):
		#master.set_timeout(600)
		#master.set_verbose(True)
		code = Demeter.service('common').one('setting_modbus_code', id=info['code_id'])
		if not code:
			return ''
		write_address = code['register_write_address'].split("\r\n")
		if not info['code_index']:
			info['code_index'] = 0
		write = write_address[info['code_index']]

		next_write = ''
		#多个用换行隔开，寄存器类型,地址,值->1||寄存器类型,地址,值->2&&寄存器类型,地址,值
		if '&&' in write:
			# 执行之后，需要执行的命令码
			temp = write.split('&&')
			write = temp[0]
			next_write = temp[1]

		if '||' in write:
			# 重新定义value
			temp = write.split('||')
			for v in temp:
				if '#' in v:
					t = v.split('#')
					if value == t[1]:
						write = t[0]

		code = self.getCode(write, code, 2)
		if 'register_length' in code and code['register_length']:
			value = code['register_length']
		result = ''
		if code['register_type']:
			#print(value)
			value = int(value)
			master.execute(int(code['slave_id']), int(code['register_type']), int(code['register_address']), output_value=value)
			result = True
			'''
			read_address = code['register_read_address'].split("\r\n")
			if info['code_index'] not in read_address:
				return True
			code = self.getCode(read_address, code)
			result = master.execute(int(code['slave_id']), int(code['register_type']), int(code['register_address']), int(code['register_length']))
			'''
			gateway = Demeter.service('common').one('device_gateway', id=info['gateway_id'])
			self.setValue(code, master, gateway)

			if next_write:
				code = self.getCode(next_write, code, 2)
				if 'register_length' in code and code['register_length']:
					value = code['register_length']
				master.execute(int(code['slave_id']), int(code['register_type']), int(code['register_address']), output_value=value)
		return result

	def setValue(self, code, master, gateway):
		config = {}
		config['parent'] = gateway['hardware_id']
		address = code['register_read_address'].split("\r\n")
		write_address = ''
		write_value = '';
		if code['register_write_address']:

			write_address = code['register_write_address'].split("\r\n")
			recv = write_address[1]
			temp = recv.split(',')
			recv_start = int(temp[0])
			recv_len = int(temp[1])

			if ',' not in write_address[0]:
				write_value = master.execute(int(code['slave_id']), write_address[0], 0, expected_length=recv_len)
				recv_start = recv_start*2
				recv_len = recv_len*2
				write_value = write_value[recv_start:recv_len]
		device_type_key = code['device_type_key'].split("\r\n")
		exp = code['exp'].split("\r\n")

		j = 0
		for i in address:
			device_type = device_type_key[j]
			device_type = Demeter.service('common').one('setting_device_type', key=device_type)
			config['type'] = device_type['key']

			info = Demeter.service('common').one('device_info', gateway_id=gateway['id'], code_id=code['id'], code_index=j)
			hardware = Demeter.service('common').one('hardware_type', id=info['hardware_type'])
			config['method'] = hardware['key']
			config['child'] = info['hardware_id']
			code = self.getCode(i, code)
			code['exp'] = exp[j]
			if write_value:
				value = self.getSourceValue(code, info, write_value)
			else:
				'''
				if write_address:
					write = write_address[j]
					write_code = self.getCode(write, {}, 2)
					if '|' in write_code['register_length']:
						temp = write_code['register_length'].split('|')
						write_value = []
						for a in temp:
							write_value.append(int(a))
					else:
						write_value = write_code['register_length']
					master.execute(int(code['slave_id']), int(write_code['register_type']), int(write_code['register_address']), output_value=write_value)
				'''

				value = self.getValue(code, master, info)
				#print(value)
			
			j = j + 1
			name = info['name']
			Demeter.service('device').save(config, value, name, device_type)

	def getSourceValue(self, code, info, source):
		register_address = int(code['register_address']) * 2
		register_length = register_address + (int(code['register_length']) * 2)
		value = source[register_address:register_length]

		value = self.hex2dec(value)
		return self.exp(code, info, value)

	def getValue(self, code, master, info):

		temp = master.execute(int(code['slave_id']), int(code['register_type']), int(code['register_address']), int(code['register_length']))
		#value = struct.unpack('>f', struct.pack('>HH', temp[0], temp[1]))
		#value = str(round(temp[0], 2))
		#value = str(temp)
		value = str(temp[0])
		return self.exp(code, info, value)

	def exp(self, code, info, value):
		if value and value != '0':
			if 'exp' in code and code['exp']:
				value = Demeter.exp(code['exp'], value)
			if 'exp' in info and info['exp'] and info['exp'] != '-1':
				value = Demeter.exp(info['exp'], value)
			value = str(round(float(value), 2))
		return value

	def hex2dec(self, string_num): 
		#start = string_num[0:1]
		#大于8为负数 后续处理吧
		return str(int(string_num.upper(), 16))

	def getCode(self, address, code, type = 1):
		(code['register_type'],code['register_address'],code['register_length']) = address.split(',')
		code['register_type'] = int(code['register_type'])
		if type == 1:
			if code['register_type'] == 1:
				# 线圈
				code['register_type'] = mdef.READ_COILS
			elif code['register_type'] == 2:
				code['register_type'] = mdef.READ_DISCRETE_INPUTS
			elif code['register_type'] == 3:
				code['register_type'] = mdef.READ_HOLDING_REGISTERS
			elif code['register_type'] == 4:
				code['register_type'] = mdef.READ_INPUT_REGISTERS
		else:
			if code['register_type'] == 1:
				# 线圈
				code['register_type'] = mdef.WRITE_SINGLE_COIL
			elif code['register_type'] == 2:
				code['register_type'] = mdef.WRITE_MULTIPLE_COILS
			elif code['register_type'] == 3:
				code['register_type'] = mdef.WRITE_SINGLE_REGISTER
			elif code['register_type'] == 4:
				code['register_type'] = mdef.WRITE_MULTIPLE_REGISTERS
		return code

	def calculate_crc(data):
		"""Calculate the CRC16 of a datagram"""
		CRC16table = (
			0x0000, 0xC0C1, 0xC181, 0x0140, 0xC301, 0x03C0, 0x0280, 0xC241,
			0xC601, 0x06C0, 0x0780, 0xC741, 0x0500, 0xC5C1, 0xC481, 0x0440,
			0xCC01, 0x0CC0, 0x0D80, 0xCD41, 0x0F00, 0xCFC1, 0xCE81, 0x0E40,
			0x0A00, 0xCAC1, 0xCB81, 0x0B40, 0xC901, 0x09C0, 0x0880, 0xC841,
			0xD801, 0x18C0, 0x1980, 0xD941, 0x1B00, 0xDBC1, 0xDA81, 0x1A40,
			0x1E00, 0xDEC1, 0xDF81, 0x1F40, 0xDD01, 0x1DC0, 0x1C80, 0xDC41,
			0x1400, 0xD4C1, 0xD581, 0x1540, 0xD701, 0x17C0, 0x1680, 0xD641,
			0xD201, 0x12C0, 0x1380, 0xD341, 0x1100, 0xD1C1, 0xD081, 0x1040,
			0xF001, 0x30C0, 0x3180, 0xF141, 0x3300, 0xF3C1, 0xF281, 0x3240,
			0x3600, 0xF6C1, 0xF781, 0x3740, 0xF501, 0x35C0, 0x3480, 0xF441,
			0x3C00, 0xFCC1, 0xFD81, 0x3D40, 0xFF01, 0x3FC0, 0x3E80, 0xFE41,
			0xFA01, 0x3AC0, 0x3B80, 0xFB41, 0x3900, 0xF9C1, 0xF881, 0x3840,
			0x2800, 0xE8C1, 0xE981, 0x2940, 0xEB01, 0x2BC0, 0x2A80, 0xEA41,
			0xEE01, 0x2EC0, 0x2F80, 0xEF41, 0x2D00, 0xEDC1, 0xEC81, 0x2C40,
			0xE401, 0x24C0, 0x2580, 0xE541, 0x2700, 0xE7C1, 0xE681, 0x2640,
			0x2200, 0xE2C1, 0xE381, 0x2340, 0xE101, 0x21C0, 0x2080, 0xE041,
			0xA001, 0x60C0, 0x6180, 0xA141, 0x6300, 0xA3C1, 0xA281, 0x6240,
			0x6600, 0xA6C1, 0xA781, 0x6740, 0xA501, 0x65C0, 0x6480, 0xA441,
			0x6C00, 0xACC1, 0xAD81, 0x6D40, 0xAF01, 0x6FC0, 0x6E80, 0xAE41,
			0xAA01, 0x6AC0, 0x6B80, 0xAB41, 0x6900, 0xA9C1, 0xA881, 0x6840,
			0x7800, 0xB8C1, 0xB981, 0x7940, 0xBB01, 0x7BC0, 0x7A80, 0xBA41,
			0xBE01, 0x7EC0, 0x7F80, 0xBF41, 0x7D00, 0xBDC1, 0xBC81, 0x7C40,
			0xB401, 0x74C0, 0x7580, 0xB541, 0x7700, 0xB7C1, 0xB681, 0x7640,
			0x7200, 0xB2C1, 0xB381, 0x7340, 0xB101, 0x71C0, 0x7080, 0xB041,
			0x5000, 0x90C1, 0x9181, 0x5140, 0x9301, 0x53C0, 0x5280, 0x9241,
			0x9601, 0x56C0, 0x5780, 0x9741, 0x5500, 0x95C1, 0x9481, 0x5440,
			0x9C01, 0x5CC0, 0x5D80, 0x9D41, 0x5F00, 0x9FC1, 0x9E81, 0x5E40,
			0x5A00, 0x9AC1, 0x9B81, 0x5B40, 0x9901, 0x59C0, 0x5880, 0x9841,
			0x8801, 0x48C0, 0x4980, 0x8941, 0x4B00, 0x8BC1, 0x8A81, 0x4A40,
			0x4E00, 0x8EC1, 0x8F81, 0x4F40, 0x8D01, 0x4DC0, 0x4C80, 0x8C41,
			0x4400, 0x84C1, 0x8581, 0x4540, 0x8701, 0x47C0, 0x4680, 0x8641,
			0x8201, 0x42C0, 0x4380, 0x8341, 0x4100, 0x81C1, 0x8081, 0x4040
		)
		crc = 0xFFFF
		if PY2:
			for c in data:
				crc = (crc >> 8) ^ CRC16table[(ord(c) ^ crc) & 0xFF]
		else:
			for c in data:
				crc = (crc >> 8) ^ CRC16table[((c) ^ crc) & 0xFF]
		return swap_bytes(crc)